@@ -36,9 +36,10 @@
 
 import copy
 import itertools
-import simplejson
 import logging
 
+import simplejson
+
 from ganeti.cli import *
 from ganeti import opcodes
 from ganeti import constants
@@ -101,7 +102,6 @@ def _ExpandMultiNames(mode, names, client=None):
   @raise errors.OpPrereqError: for invalid input parameters
 
   """
-  # pylint: disable=W0142
 
   if client is None:
     client = GetClient()
@@ -300,7 +300,7 @@ def BatchCreate(opts, args):
                                  (idx, utils.CommaJoin(unknown)),
                                  errors.ECODE_INVAL)
 
-    op = opcodes.OpInstanceCreate(**inst) # pylint: disable=W0142
+    op = opcodes.OpInstanceCreate(**inst)
     op.Validate(False)
     instances.append(op)
 
@@ -458,10 +458,11 @@ def RenameInstance(opts, args):
   @return: the desired exit code
 
   """
-  if not opts.name_check:
-    if not AskUser("As you disabled the check of the DNS entry, please verify"
-                   " that '%s' is a FQDN. Continue?" % args[1]):
-      return 1
+  if not opts.force:
+    if not opts.name_check:
+      if not AskUser("As you disabled the check of the DNS entry, please verify"
+                     " that '%s' is a FQDN. Continue?" % args[1]):
+        return 1
 
   op = opcodes.OpInstanceRename(instance_name=args[0],
                                 new_name=args[1],
@@ -736,6 +737,7 @@ def FailoverInstance(opts, args):
   """
   cl = GetClient()
   instance_name = args[0]
+  ignore_consistency = opts.ignore_consistency
   force = opts.force
   iallocator = opts.iallocator
   target_node = opts.dst_node
@@ -753,8 +755,14 @@ def FailoverInstance(opts, args):
     if not AskUser(usertext):
       return 1
 
+  if ignore_consistency:
+    usertext = ("To failover instance %s, the source node must be marked"
+                " offline first. Is this aready the case?") % instance_name
+    if not AskUser(usertext):
+      return 1
+
   op = opcodes.OpInstanceFailover(instance_name=instance_name,
-                                  ignore_consistency=opts.ignore_consistency,
+                                  ignore_consistency=ignore_consistency,
                                   shutdown_timeout=opts.shutdown_timeout,
                                   iallocator=iallocator,
                                   target_node=target_node,
@@ -877,7 +885,7 @@ def ConnectToInstanceConsole(opts, args):
 
   del cl
 
-  ((console_data, oper_state), ) = idata
+  (console_data, oper_state) = idata[0]
   if not console_data:
     if oper_state:
       # Instance is running
@@ -1398,12 +1406,7 @@ def SetInstanceParams(opts, args):
 
   # verify the user provided parameters for disk template conversions
   if opts.disk_template:
-    if (not opts.node and
-        opts.disk_template in constants.DTS_INT_MIRROR):
-      ToStderr("Changing the disk template to a mirrored one requires"
-               " specifying a secondary node")
-      return 1
-    elif (opts.ext_params and
+    if (opts.ext_params and
           opts.disk_template != constants.DT_EXT):
       ToStderr("Specifying ExtStorage parameters requires converting"
                " to the '%s' disk template" % constants.DT_EXT)
@@ -1442,6 +1445,7 @@ def SetInstanceParams(opts, args):
                                    file_driver=opts.file_driver,
                                    file_storage_dir=opts.file_storage_dir,
                                    remote_node=opts.node,
+                                   iallocator=opts.iallocator,
                                    pnode=opts.new_primary_node,
                                    hvparams=opts.hvparams,
                                    beparams=opts.beparams,
@@ -1572,7 +1576,7 @@ commands = {
   "add": (
     AddInstance, [ArgHost(min=1, max=1)],
     COMMON_CREATE_OPTS + add_opts,
-    "[...] -t disk-type -n node[:secondary-node] -o os-type <name>",
+    "[...] -t disk-type -n node[:secondary-node] -o os-type <instance-name>",
     "Creates and adds a new instance to the cluster"),
   "batch-create": (
     BatchCreate, [ArgFile(min=1, max=1)],
@@ -1582,14 +1586,15 @@ commands = {
   "console": (
     ConnectToInstanceConsole, ARGS_ONE_INSTANCE,
     [SHOWCMD_OPT, PRIORITY_OPT],
-    "[--show-cmd] <instance>", "Opens a console on the specified instance"),
+    "[--show-cmd] <instance-name>",
+    "Opens a console on the specified instance"),
   "failover": (
     FailoverInstance, ARGS_ONE_INSTANCE,
     [FORCE_OPT, IGNORE_CONSIST_OPT] + SUBMIT_OPTS +
     [SHUTDOWN_TIMEOUT_OPT,
      DRY_RUN_OPT, PRIORITY_OPT, DST_NODE_OPT, IALLOCATOR_OPT,
      IGNORE_IPOLICY_OPT, CLEANUP_OPT],
-    "[-f] <instance>", "Stops the instance, changes its primary node and"
+    "[-f] <instance-name>", "Stops the instance, changes its primary node and"
     " (if it was originally running) starts it on the new node"
     " (the secondary for mirrored instances or any node"
     " for shared storage)."),
@@ -1599,7 +1604,7 @@ commands = {
      PRIORITY_OPT, DST_NODE_OPT, IALLOCATOR_OPT, ALLOW_FAILOVER_OPT,
      IGNORE_IPOLICY_OPT, IGNORE_HVVERSIONS_OPT, NORUNTIME_CHGS_OPT]
     + SUBMIT_OPTS,
-    "[-f] <instance>", "Migrate instance to its secondary node"
+    "[-f] <instance-name>", "Migrate instance to its secondary node"
     " (only for mirrored instances)"),
   "move": (
     MoveInstance, ARGS_ONE_INSTANCE,
@@ -1607,18 +1612,18 @@ commands = {
     [SINGLE_NODE_OPT, COMPRESS_OPT,
      SHUTDOWN_TIMEOUT_OPT, DRY_RUN_OPT, PRIORITY_OPT, IGNORE_CONSIST_OPT,
      IGNORE_IPOLICY_OPT],
-    "[-f] <instance>", "Move instance to an arbitrary node"
+    "[-f] <instance-name>", "Move instance to an arbitrary node"
     " (only for instances of type file and lv)"),
   "info": (
     ShowInstanceConfig, ARGS_MANY_INSTANCES,
     [STATIC_OPT, ALL_OPT, ROMAN_OPT, PRIORITY_OPT],
-    "[-s] {--all | <instance>...}",
+    "[-s] {--all | <instance-name>...}",
     "Show information on the specified instance(s)"),
   "list": (
     ListInstances, ARGS_MANY_INSTANCES,
     [NOHDR_OPT, SEP_OPT, USEUNITS_OPT, FIELDS_OPT, VERBOSE_OPT,
      FORCE_FILTER_OPT],
-    "[<instance>...]",
+    "[<instance-name>...]",
     "Lists the instances and their status. The available fields can be shown"
     " using the \"list-fields\" command (see the man page for details)."
     " The default field list is (in order): %s." %
@@ -1641,30 +1646,31 @@ commands = {
     RemoveInstance, ARGS_ONE_INSTANCE,
     [FORCE_OPT, SHUTDOWN_TIMEOUT_OPT, IGNORE_FAILURES_OPT] + SUBMIT_OPTS
     + [DRY_RUN_OPT, PRIORITY_OPT],
-    "[-f] <instance>", "Shuts down the instance and removes it"),
+    "[-f] <instance-name>", "Shuts down the instance and removes it"),
   "rename": (
     RenameInstance,
     [ArgInstance(min=1, max=1), ArgHost(min=1, max=1)],
-    [NOIPCHECK_OPT, NONAMECHECK_OPT] + SUBMIT_OPTS
+    [FORCE_OPT, NOIPCHECK_OPT, NONAMECHECK_OPT] + SUBMIT_OPTS
     + [DRY_RUN_OPT, PRIORITY_OPT],
-    "<instance> <new_name>", "Rename the instance"),
+    "<old-name> <new-name>", "Rename the instance"),
   "replace-disks": (
     ReplaceDisks, ARGS_ONE_INSTANCE,
     [AUTO_REPLACE_OPT, DISKIDX_OPT, IALLOCATOR_OPT, EARLY_RELEASE_OPT,
      NEW_SECONDARY_OPT, ON_PRIMARY_OPT, ON_SECONDARY_OPT] + SUBMIT_OPTS
     + [DRY_RUN_OPT, PRIORITY_OPT, IGNORE_IPOLICY_OPT],
-    "[-s|-p|-a|-n NODE|-I NAME] <instance>",
+    "[-s|-p|-a|-n NODE|-I NAME] <instance-name>",
     "Replaces disks for the instance"),
   "modify": (
     SetInstanceParams, ARGS_ONE_INSTANCE,
     [BACKEND_OPT, DISK_OPT, FORCE_OPT, HVOPTS_OPT, NET_OPT] + SUBMIT_OPTS +
-    [DISK_TEMPLATE_OPT, SINGLE_NODE_OPT, OS_OPT, FORCE_VARIANT_OPT,
+    [DISK_TEMPLATE_OPT, SINGLE_NODE_OPT, IALLOCATOR_OPT,
+     OS_OPT, FORCE_VARIANT_OPT,
      OSPARAMS_OPT, OSPARAMS_PRIVATE_OPT, DRY_RUN_OPT, PRIORITY_OPT, NWSYNC_OPT,
      OFFLINE_INST_OPT, ONLINE_INST_OPT, IGNORE_IPOLICY_OPT, RUNTIME_MEM_OPT,
      NOCONFLICTSCHECK_OPT, NEW_PRIMARY_OPT, HOTPLUG_OPT,
      HOTPLUG_IF_POSSIBLE_OPT, INSTANCE_COMMUNICATION_OPT,
      EXT_PARAMS_OPT, FILESTORE_DRIVER_OPT, FILESTORE_DIR_OPT],
-    "<instance>", "Alters the parameters of an instance"),
+    "<instance-name>", "Alters the parameters of an instance"),
   "shutdown": (
     GenericManyOps("shutdown", _ShutdownInstance), [ArgInstance()],
     [FORCE_OPT, m_node_opt, m_pri_node_opt, m_sec_node_opt, m_clust_opt,
@@ -1691,40 +1697,41 @@ commands = {
   "activate-disks": (
     ActivateDisks, ARGS_ONE_INSTANCE,
     SUBMIT_OPTS + [IGNORE_SIZE_OPT, PRIORITY_OPT, WFSYNC_OPT],
-    "<instance>", "Activate an instance's disks"),
+    "<instance-name>", "Activate an instance's disks"),
   "deactivate-disks": (
     DeactivateDisks, ARGS_ONE_INSTANCE,
     [FORCE_OPT] + SUBMIT_OPTS + [DRY_RUN_OPT, PRIORITY_OPT],
-    "[-f] <instance>", "Deactivate an instance's disks"),
+    "[-f] <instance-name>", "Deactivate an instance's disks"),
   "recreate-disks": (
     RecreateDisks, ARGS_ONE_INSTANCE,
     SUBMIT_OPTS +
     [DISK_OPT, NODE_PLACEMENT_OPT, DRY_RUN_OPT, PRIORITY_OPT,
      IALLOCATOR_OPT],
-    "<instance>", "Recreate an instance's disks"),
+    "<instance-name>", "Recreate an instance's disks"),
   "grow-disk": (
     GrowDisk,
     [ArgInstance(min=1, max=1), ArgUnknown(min=1, max=1),
      ArgUnknown(min=1, max=1)],
     SUBMIT_OPTS +
     [NWSYNC_OPT, DRY_RUN_OPT, PRIORITY_OPT, ABSOLUTE_OPT, IGNORE_IPOLICY_OPT],
-    "<instance> <disk> <size>", "Grow an instance's disk"),
+    "<instance-name> <disk> <size>", "Grow an instance's disk"),
   "change-group": (
     ChangeGroup, ARGS_ONE_INSTANCE,
     [TO_GROUP_OPT, IALLOCATOR_OPT, EARLY_RELEASE_OPT, PRIORITY_OPT]
     + SUBMIT_OPTS,
-    "[-I <iallocator>] [--to <group>]", "Change group of instance"),
+    "[-I <iallocator>] [--to <group>] <instance-name>",
+    "Change group of instance"),
   "list-tags": (
     ListTags, ARGS_ONE_INSTANCE, [],
-    "<instance_name>", "List the tags of the given instance"),
+    "<instance-name>", "List the tags of the given instance"),
   "add-tags": (
     AddTags, [ArgInstance(min=1, max=1), ArgUnknown()],
     [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<instance_name> tag...", "Add tags to the given instance"),
+    "<instance-name> <tag>...", "Add tags to the given instance"),
   "remove-tags": (
     RemoveTags, [ArgInstance(min=1, max=1), ArgUnknown()],
     [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<instance_name> tag...", "Remove tags from given instance"),
+    "<instance-name> <tag>...", "Remove tags from given instance"),
   }
 
 #: dictionary with aliases for commands
