@@ -47,6 +47,7 @@ from ganeti import constants
 from ganeti import errors
 from ganeti import netutils
 from ganeti import pathutils
+from ganeti.rpc.node import RunWithRPC
 from ganeti import ssh
 from ganeti import compat
 
@@ -230,12 +231,17 @@ def _SetupSSH(options, cluster_name, node, ssh_port, cl):
   (_, cert_pem) = \
     utils.ExtractX509Certificate(utils.ReadFile(pathutils.NODED_CERT_FILE))
 
+  (ssh_key_type, ssh_key_bits) = \
+    cl.QueryConfigValues(["ssh_key_type", "ssh_key_bits"])
+
   data = {
     constants.SSHS_CLUSTER_NAME: cluster_name,
     constants.SSHS_NODE_DAEMON_CERTIFICATE: cert_pem,
     constants.SSHS_SSH_HOST_KEY: host_keys,
     constants.SSHS_SSH_ROOT_KEY: root_keys,
     constants.SSHS_SSH_AUTHORIZED_KEYS: candidate_keys,
+    constants.SSHS_SSH_KEY_TYPE: ssh_key_type,
+    constants.SSHS_SSH_KEY_BITS: ssh_key_bits,
     }
 
   ssh.RunSshCmdWithStdin(cluster_name, node, pathutils.PREPARE_NODE_JOIN,
@@ -244,9 +250,9 @@ def _SetupSSH(options, cluster_name, node, ssh_port, cl):
                          use_cluster_key=False, ask_key=options.ssh_key_check,
                          strict_host_check=options.ssh_key_check)
 
-  (_, dsa_pub_keyfile) = root_keyfiles[constants.SSHK_DSA]
-  pub_key = ssh.ReadRemoteSshPubKeys(dsa_pub_keyfile, node, cluster_name,
-                                     ssh_port, options.ssh_key_check,
+  (_, pub_keyfile) = root_keyfiles[ssh_key_type]
+  pub_key = ssh.ReadRemoteSshPubKeys(pub_keyfile, node, cluster_name, ssh_port,
+                                     options.ssh_key_check,
                                      options.ssh_key_check)
   # Unfortunately, we have to add the key with the node name rather than
   # the node's UUID here, because at this point, we do not have a UUID yet.
@@ -254,7 +260,7 @@ def _SetupSSH(options, cluster_name, node, ssh_port, cl):
   ssh.AddPublicKey(node, pub_key)
 
 
-@UsesRPC
+@RunWithRPC
 def AddNode(opts, args):
   """Add a node to the cluster.
 
@@ -312,7 +318,17 @@ def AddNode(opts, args):
   # read the cluster name from the master
   (cluster_name, ) = cl.QueryConfigValues(["cluster_name"])
 
-  if opts.node_setup:
+  if not opts.node_setup:
+    ToStdout("-- WARNING -- \n"
+             "The option --no-node-setup is disabled. Whether or not the\n"
+             "SSH setup is manipulated while adding a node is determined\n"
+             "by the 'modify_ssh_setup' value in the cluster-wide\n"
+             "configuration instead.\n")
+
+  (modify_ssh_setup, ) = \
+    cl.QueryConfigValues(["modify_ssh_setup"])
+
+  if modify_ssh_setup:
     ToStderr("-- WARNING -- \n"
              "Performing this operation is going to perform the following\n"
              "changes to the target machine (%s) and the current cluster\n"
@@ -324,7 +340,7 @@ def AddNode(opts, args):
              "  generated public SSH key will be distributed to all other\n"
              "  cluster nodes.\n", node)
 
-  if opts.node_setup:
+  if modify_ssh_setup:
     _SetupSSH(opts, cluster_name, node, ssh_port, cl)
 
   bootstrap.SetupNodeDaemon(opts, cluster_name, node, ssh_port)
@@ -342,7 +358,7 @@ def AddNode(opts, args):
                          master_capable=opts.master_capable,
                          disk_state=disk_state,
                          hv_state=hv_state,
-                         node_setup=opts.node_setup)
+                         node_setup=modify_ssh_setup)
   SubmitOpCode(op, opts=opts)
 
 
@@ -972,6 +988,18 @@ def SetNodeParams(opts, args):
   else:
     disk_state = {}
 
+  # Comparing explicitly to false to distinguish between a parameter
+  # modification that doesn't set the node online (where the value will be None)
+  # and modifying the node to bring it online.
+  if opts.offline is False and not opts.force:
+    usertext = ("You are setting this node online manually. If the"
+                " configuration has changed, this can cause issues such as"
+                " split brain. To safely bring a node back online, please use"
+                " --readd instead. If you are confident that the configuration"
+                " hasn't changed, continue?")
+    if not AskUser(usertext):
+      return 1
+
   hv_state = dict(opts.hv_state)
 
   op = opcodes.OpNodeSetParams(node_name=args[0],
@@ -1133,7 +1161,7 @@ commands = {
      CAPAB_MASTER_OPT, CAPAB_VM_OPT, NODE_PARAMS_OPT, HV_STATE_OPT,
      DISK_STATE_OPT],
     "[-s ip] [--readd] [--no-ssh-key-check] [--force-join]"
-    " [--no-node-setup] [--verbose] [--network] <node_name>",
+    " [--no-node-setup] [--verbose] [--network] <node-name>",
     "Add a node to the cluster"),
   "evacuate": (
     EvacuateNode, ARGS_ONE_NODE,
@@ -1158,12 +1186,12 @@ commands = {
     " (only for instances of type drbd)"),
   "info": (
     ShowNodeConfig, ARGS_MANY_NODES, [],
-    "[<node_name>...]", "Show information about the node(s)"),
+    "[<node>...]", "Show information about the node(s)"),
   "list": (
     ListNodes, ARGS_MANY_NODES,
     [NOHDR_OPT, SEP_OPT, USEUNITS_OPT, FIELDS_OPT, VERBOSE_OPT,
      FORCE_FILTER_OPT],
-    "[nodes...]",
+    "[<node-name>...]",
     "Lists the nodes in the cluster. The available fields can be shown using"
     " the \"list-fields\" command (see the man page for details)."
     " The default field list is (in order): %s." %
@@ -1180,11 +1208,11 @@ commands = {
      CAPAB_MASTER_OPT, CAPAB_VM_OPT, SECONDARY_IP_OPT,
      AUTO_PROMOTE_OPT, DRY_RUN_OPT, PRIORITY_OPT, NODE_PARAMS_OPT,
      NODE_POWERED_OPT, HV_STATE_OPT, DISK_STATE_OPT],
-    "<node_name>", "Alters the parameters of a node"),
+    "<node-name>", "Alters the parameters of a node"),
   "powercycle": (
     PowercycleNode, ARGS_ONE_NODE,
     [FORCE_OPT, CONFIRM_OPT, DRY_RUN_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<node_name>", "Tries to forcefully powercycle a node"),
+    "<node-name>", "Tries to forcefully powercycle a node"),
   "power": (
     PowerNode,
     [ArgChoice(min=1, max=1, choices=_LIST_POWER_COMMANDS),
@@ -1193,20 +1221,20 @@ commands = {
     [AUTO_PROMOTE_OPT, PRIORITY_OPT,
      IGNORE_STATUS_OPT, FORCE_OPT, NOHDR_OPT, SEP_OPT, OOB_TIMEOUT_OPT,
      POWER_DELAY_OPT],
-    "on|off|cycle|status [nodes...]",
+    "on|off|cycle|status [<node-name>...]",
     "Change power state of node by calling out-of-band helper."),
   "remove": (
     RemoveNode, ARGS_ONE_NODE, [DRY_RUN_OPT, PRIORITY_OPT],
-    "<node_name>", "Removes a node from the cluster"),
+    "<node-name>", "Removes a node from the cluster"),
   "volumes": (
     ListVolumes, [ArgNode()],
     [NOHDR_OPT, SEP_OPT, USEUNITS_OPT, FIELDS_OPT, PRIORITY_OPT],
-    "[<node_name>...]", "List logical volumes on node(s)"),
+    "[<node-name>...]", "List logical volumes on node(s)"),
   "list-storage": (
     ListStorage, ARGS_MANY_NODES,
     [NOHDR_OPT, SEP_OPT, USEUNITS_OPT, FIELDS_OPT, _STORAGE_TYPE_OPT,
      PRIORITY_OPT],
-    "[<node_name>...]", "List physical volumes on node(s). The available"
+    "[<node-name>...]", "List physical volumes on node(s). The available"
     " fields are (see the man page for details): %s." %
     (utils.CommaJoin(_LIST_STOR_HEADERS))),
   "modify-storage": (
@@ -1215,38 +1243,39 @@ commands = {
      ArgChoice(min=1, max=1, choices=_MODIFIABLE_STORAGE_TYPES),
      ArgFile(min=1, max=1)],
     [ALLOCATABLE_OPT, DRY_RUN_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<node_name> <storage_type> <name>", "Modify storage volume on a node"),
+    "<node-name> <storage-type> <volume-name>",
+    "Modify storage volume on a node"),
   "repair-storage": (
     RepairStorage,
     [ArgNode(min=1, max=1),
      ArgChoice(min=1, max=1, choices=_REPAIRABLE_STORAGE_TYPES),
      ArgFile(min=1, max=1)],
     [IGNORE_CONSIST_OPT, DRY_RUN_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<node_name> <storage_type> <name>",
+    "<node-name> <storage-type> <volume-name>",
     "Repairs a storage volume on a node"),
   "list-tags": (
     ListTags, ARGS_ONE_NODE, [],
-    "<node_name>", "List the tags of the given node"),
+    "<node-name>", "List the tags of the given node"),
   "add-tags": (
     AddTags, [ArgNode(min=1, max=1), ArgUnknown()],
     [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<node_name> tag...", "Add tags to the given node"),
+    "<node-name> <tag>...", "Add tags to the given node"),
   "remove-tags": (
     RemoveTags, [ArgNode(min=1, max=1), ArgUnknown()],
     [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "<node_name> tag...", "Remove tags from the given node"),
+    "<node-name> <tag>...", "Remove tags from the given node"),
   "health": (
     Health, ARGS_MANY_NODES,
     [NOHDR_OPT, SEP_OPT, PRIORITY_OPT, OOB_TIMEOUT_OPT],
-    "[<node_name>...]", "List health of node(s) using out-of-band"),
+    "[<node-name>...]", "List health of node(s) using out-of-band"),
   "list-drbd": (
     ListDrbd, ARGS_ONE_NODE,
     [NOHDR_OPT, SEP_OPT],
-    "[<node_name>]", "Query the list of used DRBD minors on the given node"),
+    "<node>", "Query the list of used DRBD minors on the given node"),
   "restricted-command": (
     RestrictedCommand, [ArgUnknown(min=1, max=1)] + ARGS_MANY_NODES,
     [SYNC_OPT, PRIORITY_OPT] + SUBMIT_OPTS + [SHOW_MACHINE_OPT, NODEGROUP_OPT],
-    "<command> <node_name> [<node_name>...]",
+    "<command> <node-name>...",
     "Executes a restricted command on node(s)"),
   }
 
