@@ -32,10 +32,12 @@
 """
 
 import logging
-import pycurl
 import threading
+
 from cStringIO import StringIO
 
+import pycurl
+
 from ganeti import http
 from ganeti import compat
 from ganeti import netutils
@@ -225,7 +227,15 @@ class _PendingRequest(object):
 
     assert req.success is None, "Request has already been finalized"
 
-    logging.debug("Request %s finished, errmsg=%s", req, errmsg)
+    try:
+      # LOCAL_* options added in pycurl 7.21.5
+      from_str = "from %s:%s " % (
+          curl.getinfo(pycurl.LOCAL_IP),
+          curl.getinfo(pycurl.LOCAL_PORT)
+      )
+    except AttributeError:
+      from_str = ""
+    logging.debug("Request %s%s finished, errmsg=%s", from_str, req, errmsg)
 
     req.success = not bool(errmsg)
     req.error = errmsg
@@ -371,7 +381,7 @@ def ProcessRequests(requests, lock_monitor_cb=None, _curl=pycurl.Curl,
   # Prepare all requests
   curl_to_client = \
     dict((client.GetCurlHandle(), client)
-         for client in map(lambda req: _StartRequest(_curl(), req), requests))
+         for client in [_StartRequest(_curl(), req) for req in requests])
 
   assert len(curl_to_client) == len(requests)
 
