@@ -52,7 +52,6 @@ from ganeti import hooksmaster
 from ganeti import cmdlib
 from ganeti import locking
 from ganeti import utils
-from ganeti import compat
 from ganeti import wconfd
 
 
@@ -158,6 +157,7 @@ class OpExecCbBase(object): # pylint: disable=W0232
 
     """
 
+  # TODO: Cleanup calling conventions, make them explicit.
   def Feedback(self, *args):
     """Sends feedback from the LU code to the end-user.
 
@@ -231,9 +231,8 @@ def _ProcessResult(submit_fn, op, result):
   """
   if isinstance(result, cmdlib.ResultWithJobs):
     # Copy basic parameters (e.g. priority)
-    map(compat.partial(_SetBaseOpParams, op,
-                       "Submitted by %s" % op.OP_ID),
-        itertools.chain(*result.jobs))
+    for op2 in itertools.chain(*result.jobs):
+      _SetBaseOpParams(op, "Submitted by %s" % op.OP_ID, op2)
 
     # Submit jobs
     job_submission = submit_fn(result.jobs)
@@ -276,6 +275,17 @@ def _LockList(names):
     return list(names)
 
 
+def _CheckSecretParameters(op):
+  """Check if secret parameters are expected, but missing.
+
+  """
+  if hasattr(op, "osparams_secret") and op.osparams_secret:
+    for secret_param in op.osparams_secret:
+      if op.osparams_secret[secret_param].Get() == constants.REDACTED:
+        raise errors.OpPrereqError("Please re-submit secret parameters to job.",
+                                   errors.ECODE_INVAL)
+
+
 class Processor(object):
   """Object which runs OpCodes"""
   DISPATCH_TABLE = _ComputeDispatchTable()
@@ -289,7 +299,6 @@ class Processor(object):
     @param ec_id: execution context identifier
 
     """
-    self.context = context
     self._ec_id = ec_id
     self._cbs = None
     self.cfg = context.GetConfig(ec_id)
@@ -537,9 +546,9 @@ class Processor(object):
       try:
         result = self._ExecLU(lu)
       except errors.OpPrereqError, err:
-        (_, ecode) = err.args
-        if ecode != errors.ECODE_TEMP_NORES:
+        if len(err.args) < 2 or err.args[1] != errors.ECODE_TEMP_NORES:
           raise
+
         logging.debug("Temporarily out of resources; will retry internally")
         try:
           lu.PrepareRetry(self.Log)
@@ -685,9 +694,10 @@ class Processor(object):
         raise errors.ProgrammerError("Opcode '%s' requires BGL, but locks are"
                                      " disabled" % op.OP_ID)
 
-      lu = lu_class(self, op, self.context, self.cfg, self.rpc,
+      lu = lu_class(self, op, self.cfg, self.rpc,
                     self._wconfdcontext, self.wconfd)
       lu.wconfdlocks = self.wconfd.Client().ListLocks(self._wconfdcontext)
+      _CheckSecretParameters(op)
       lu.ExpandNames()
       assert lu.needed_locks is not None, "needed_locks not set by LU"
 
