@@ -46,7 +46,6 @@ module Ganeti.HTools.Node
   , setIdx
   , setAlias
   , setOffline
-  , setXmem
   , setPri
   , calcFmemOfflineOrForthcoming
   , setSec
@@ -59,6 +58,7 @@ module Ganeti.HTools.Node
   , setMigrationTags
   , setRecvMigrationTags
   , setLocationTags
+  , setHypervisor
   -- * Tag maps
   , addTags
   , delTags
@@ -77,8 +77,10 @@ module Ganeti.HTools.Node
   -- * Stats
   , availDisk
   , availMem
+  , missingMem
+  , unallocatedMem
+  , recordedFreeMem
   , availCpu
-  , iMem
   , iDsk
   , conflictingPrimaries
   -- * Generate OpCodes
@@ -92,8 +94,6 @@ module Ganeti.HTools.Node
   , list
   -- * Misc stuff
   , AssocList
-  , GenericAllocElement
-  , AllocElement
   , noSecondary
   , computeGroups
   , mkNodeGraph
@@ -115,7 +115,7 @@ import Text.Printf (printf)
 
 import qualified Ganeti.Constants as C
 import qualified Ganeti.OpCodes as OpCodes
-import Ganeti.Types (OobCommand(..), TagKind(..), mkNonEmpty)
+import Ganeti.Types (Hypervisor(..), OobCommand(..), TagKind(..), mkNonEmpty)
 import Ganeti.HTools.Container (Container)
 import qualified Ganeti.HTools.Container as Container
 import Ganeti.HTools.Instance (Instance)
@@ -134,11 +134,13 @@ type TagMap = Map.Map String Int
 data Node = Node
   { name     :: String    -- ^ The node name
   , alias    :: String    -- ^ The shortened name (for display purposes)
-  , tMem     :: Double    -- ^ Total memory (MiB)
-  , nMem     :: Int       -- ^ Node memory (MiB)
-  , fMem     :: Int       -- ^ Free memory (MiB)
+  , tMem     :: Double    -- ^ Total memory (MiB) (state-of-world)
+  , nMem     :: Int       -- ^ Node memory (MiB) (state-of-record)
+  , iMem     :: Int       -- ^ Instance memory (MiB) (state-of-record)
+  , fMem     :: Int       -- ^ Free memory (MiB) (state-of-world)
   , fMemForth :: Int      -- ^ Free memory (MiB) including forthcoming
-                          --   instances
+                          --   instances TODO: Use state of record calculations
+                          --   for forthcoming instances (see unallocatedMem)
   , xMem     :: Int       -- ^ Unaccounted memory (MiB)
   , tDsk     :: Double    -- ^ Total disk space (MiB)
   , fDsk     :: Int       -- ^ Free disk space (MiB)
@@ -210,7 +212,12 @@ data Node = Node
   , rmigTags :: Set.Set String -- ^ migration tags able to receive
   , locationTags :: Set.Set String -- ^ common-failure domains the node belongs
                                    -- to
-  , locationScore :: Int
+  , locationScore :: Int -- ^ Sum of instance location and desired location
+                         -- scores
+  , instanceMap :: Map.Map (String, String) Int -- ^ Number of instances with
+                                                -- each exclusion/location tags
+                                                -- pair
+  , hypervisor :: Maybe Hypervisor -- ^ Active hypervisor on the node
   } deriving (Show, Eq)
 {- A note on how we handle spindles
 
@@ -246,37 +253,29 @@ type AssocList = [(T.Ndx, Node)]
 -- | A simple name for a node map.
 type List = Container.Container Node
 
--- | A simple name for an allocation element (here just for logistic
--- reasons), generic in the type of the metric.
-type GenericAllocElement a = (List, Instance.Instance, [Node], a)
-
--- | A simple name for an allocation element (here just for logistic
--- reasons).
-type AllocElement = GenericAllocElement T.Score
-
 -- | Constant node index for a non-moveable instance.
 noSecondary :: T.Ndx
 noSecondary = -1
 
 -- * Helper functions
 
--- | Add a tag to a tagmap.
-addTag :: TagMap -> String -> TagMap
+-- | Add a value to a map.
+addTag :: (Ord k) => Map.Map k Int -> k -> Map.Map k Int
 addTag t s = Map.insertWith (+) s 1 t
 
--- | Add multiple tags.
-addTags :: TagMap -> [String] -> TagMap
+-- | Add multiple values.
+addTags :: (Ord k) => Map.Map k Int -> [k] -> Map.Map k Int
 addTags = foldl' addTag
 
--- | Adjust or delete a tag from a tagmap.
-delTag :: TagMap -> String -> TagMap
+-- | Adjust or delete a value from a map.
+delTag :: (Ord k) => Map.Map k Int -> k -> Map.Map k Int
 delTag t s = Map.update (\v -> if v > 1
                                  then Just (v-1)
                                  else Nothing)
              s t
 
--- | Remove multiple tags.
-delTags :: TagMap -> [String] -> TagMap
+-- | Remove multiple value.
+delTags :: (Ord k) => Map.Map k Int -> [k] -> Map.Map k Int
 delTags = foldl' delTag
 
 -- | Check if we can add a list of tags to a tagmap.
@@ -325,6 +324,9 @@ create name_init mem_t_init mem_n_init mem_f_init
        , alias = name_init
        , tMem = mem_t_init
        , nMem = mem_n_init
+       , iMem = 0 -- updated after instances are loaded
+       , xMem = 0 -- updated after instances are loaded
+       , pMem = 0 -- updated after instances are loaded
        , fMem = mem_f_init
        , fMemForth = mem_f_init
        , tDsk = dsk_t_init
@@ -348,7 +350,6 @@ create name_init mem_t_init mem_n_init mem_f_init
        , peers = P.empty
        , rMem = 0
        , rMemForth = 0
-       , pMem = fromIntegral mem_f_init / mem_t_init
        , pMemForth = fromIntegral mem_f_init / mem_t_init
        , pDsk = if excl_stor
                 then computePDsk spindles_f_init $ fromIntegral spindles_t_init
@@ -364,7 +365,6 @@ create name_init mem_t_init mem_n_init mem_f_init
        , offline = offline_init
        , isMaster = False
        , nTags = []
-       , xMem = 0
        , mDsk = T.defReservedDiskRatio
        , loDsk = mDskToloDsk T.defReservedDiskRatio dsk_t_init
        , hiCpu = mCpuTohiCpu (T.iPolicyVcpuRatio T.defIPolicy) cpu_t_init
@@ -383,6 +383,8 @@ create name_init mem_t_init mem_n_init mem_f_init
        , rmigTags = Set.empty
        , locationTags = Set.empty
        , locationScore = 0
+       , instanceMap = Map.empty
+       , hypervisor = Nothing
        }
 
 -- | Conversion formula from mDsk\/tDsk to loDsk.
@@ -433,9 +435,9 @@ setRecvMigrationTags t val = t { rmigTags = val }
 setLocationTags :: Node -> Set.Set String -> Node
 setLocationTags t val = t { locationTags = val }
 
--- | Sets the unnaccounted memory.
-setXmem :: Node -> Int -> Node
-setXmem t val = t { xMem = val }
+-- | Sets the hypervisor attribute.
+setHypervisor :: Node -> Hypervisor -> Node
+setHypervisor t val = t { hypervisor = Just val }
 
 -- | Sets the max disk usage ratio.
 setMdsk :: Node -> Double -> Node
@@ -544,8 +546,28 @@ calcFmemOfflineOrForthcoming node allInstances =
          . filter (not . Instance.usesMemory)
          $ nodeInstances
 
+-- | Calculates the desired location score of an instance, given its primary
+-- node.
+getInstanceDsrdLocScore :: Node -- ^ the primary node of the instance
+                        -> Instance.Instance -- ^ the original instance
+                        -> Int -- ^ the desired location score of the instance
+getInstanceDsrdLocScore p t =
+        desiredLocationScore (Instance.dsrdLocTags t) (locationTags p)
+  where desiredLocationScore instTags nodeTags =
+          Set.size instTags - Set.size ( instTags `Set.intersection` nodeTags )
+        -- this way we get the number of unsatisfied desired locations
+
+-- | Returns list of all pairs of node location and instance
+-- exclusion tags.
+getLocationExclusionPairs :: Node -- ^ the primary node of the instance
+                           -> Instance.Instance -- ^ the instance
+                           -> [(String, String)]
+getLocationExclusionPairs p inst =
+  [(loc, excl) | loc <- Set.toList (locationTags p)
+               , excl <- Instance.exclTags inst]
+
 -- | Assigns an instance to a node as primary and update the used VCPU
--- count, utilisation data and tags map.
+-- count, utilisation data, tags map and desired location score.
 setPri :: Node -> Instance.Instance -> Node
 setPri t inst
   -- Real instance, update real fields and forthcoming fields.
@@ -557,6 +579,8 @@ setPri t inst
           , utilLoad = utilLoad t `T.addUtil` Instance.util inst
           , instSpindles = calcSpindleUse True t inst
           , locationScore = locationScore t + Instance.locationScore inst
+                            + getInstanceDsrdLocScore t inst
+          , instanceMap = new_instance_map
           }
 
   -- Forthcoming instance, update forthcoming fields only.
@@ -566,6 +590,8 @@ setPri t inst
     new_count = Instance.applyIfOnline inst (+ Instance.vcpus inst) (uCpu t)
     new_count_forth = Instance.applyIfOnline inst (+ Instance.vcpus inst)
                                              (uCpuForth t)
+    new_instance_map = addTags (instanceMap t)
+                     $ getLocationExclusionPairs t inst
 
     uses_disk = Instance.usesLocalStorage inst
 
@@ -735,25 +761,32 @@ removePri t inst =
        then updateForthcomingFields t
        else let
                 new_plist = delete iname (pList t)
-                new_mem = incIf (Instance.usesMemory inst) (fMem t)
-                                (Instance.mem inst)
+
+                (new_i_mem, new_free_mem) = prospectiveMem t inst False
+                new_p_mem = fromIntegral new_free_mem / tMem t
+                new_failn1 = new_free_mem <= rMem t
+
                 new_dsk = incIf uses_disk (fDsk t) (Instance.dsk inst)
                 new_free_sp = calcNewFreeSpindles False t inst
                 new_inst_sp = calcSpindleUse False t inst
-                new_mp = fromIntegral new_mem / tMem t
                 new_dp = computeNewPDsk t new_free_sp new_dsk
-                new_failn1 = new_mem <= rMem t
+
                 new_ucpu = decIf i_online (uCpu t) (Instance.vcpus inst)
                 new_rcpu = fromIntegral new_ucpu / tCpu t
                 new_load = utilLoad t `T.subUtil` Instance.util inst
 
+                new_instance_map = delTags (instanceMap t)
+                                 $ getLocationExclusionPairs t inst
+
             in updateForthcomingFields $
-                 t { pList = new_plist, fMem = new_mem, fDsk = new_dsk
-                   , failN1 = new_failn1, pMem = new_mp, pDsk = new_dp
+                 t { pList = new_plist, iMem = new_i_mem, fDsk = new_dsk
+                   , failN1 = new_failn1, pMem = new_p_mem, pDsk = new_dp
                    , uCpu = new_ucpu, pCpu = new_rcpu, utilLoad = new_load
                    , instSpindles = new_inst_sp, fSpindles = new_free_sp
                    , locationScore = locationScore t
                                      - Instance.locationScore inst
+                                     - getInstanceDsrdLocScore t inst
+                   , instanceMap = new_instance_map
                    }
 
 -- | Removes a secondary instance.
@@ -806,7 +839,7 @@ removeSec t inst =
                              then old_rmem
                              else computeMaxRes new_peers
                 new_prem = fromIntegral new_rmem / tMem t
-                new_failn1 = fMem t <= new_rmem
+                new_failn1 = unallocatedMem t <= new_rmem
                 new_dp = computeNewPDsk t new_free_sp new_dsk
                 old_load = utilLoad t
                 new_load = old_load
@@ -843,7 +876,6 @@ addPriEx force t inst =
       l_cpu = T.iPolicyVcpuRatio $ iPolicy t
       old_tags = pTags t
       strict = not force
-
       inst_tags = Instance.exclTags inst
 
       new_mem_forth = fMemForth t - Instance.mem inst
@@ -893,21 +925,23 @@ addPriEx force t inst =
              _ -> Ok $ updateForthcomingFields t
 
       else let
-               new_mem = decIf (Instance.usesMemory inst) (fMem t)
-                               (Instance.mem inst)
+               (new_i_mem, new_free_mem) = prospectiveMem t inst True
+               new_p_mem = fromIntegral new_free_mem / tMem t
+               new_failn1 = new_free_mem <= rMem t
+
                new_dsk = decIf uses_disk (fDsk t) (Instance.dsk inst)
                new_free_sp = calcNewFreeSpindles True t inst
                new_inst_sp = calcSpindleUse True t inst
-               new_failn1 = new_mem <= rMem t
                new_ucpu = incIf i_online (uCpu t) (Instance.vcpus inst)
                new_pcpu = fromIntegral new_ucpu / tCpu t
                new_dp = computeNewPDsk t new_free_sp new_dsk
                new_load = utilLoad t `T.addUtil` Instance.util inst
 
                new_plist = iname:pList t
-               new_mp = fromIntegral new_mem / tMem t
+               new_instance_map = addTags (instanceMap t)
+                                $ getLocationExclusionPairs t inst
       in case () of
-        _ | new_mem <= 0 -> Bad T.FailMem
+        _ | new_free_mem <= 0 -> Bad T.FailMem
           | uses_disk && new_dsk <= 0 -> Bad T.FailDisk
           | strict && uses_disk && new_dsk < loDsk t -> Bad T.FailDisk
           | uses_disk && exclStorage t && new_free_sp < 0 -> Bad T.FailSpindles
@@ -922,10 +956,10 @@ addPriEx force t inst =
           | otherwise ->
               Ok . updateForthcomingFields $
                 t { pList = new_plist
-                  , fMem = new_mem
+                  , iMem = new_i_mem
                   , fDsk = new_dsk
                   , failN1 = new_failn1
-                  , pMem = new_mp
+                  , pMem = new_p_mem
                   , pDsk = new_dp
                   , uCpu = new_ucpu
                   , pCpu = new_pcpu
@@ -934,6 +968,8 @@ addPriEx force t inst =
                   , fSpindles = new_free_sp
                   , locationScore = locationScore t
                                     + Instance.locationScore inst
+                                    + getInstanceDsrdLocScore t inst
+                  , instanceMap = new_instance_map
                   }
 
 -- | Adds a secondary instance (basic version).
@@ -1001,7 +1037,7 @@ addSecExEx ignore_disks force t inst pdx =
 
              _ -> Ok $ updateForthcomingFields t
       else let
-               old_mem = fMem t
+               old_mem = unallocatedMem t
                new_dsk = fDsk t - Instance.dsk inst
                new_free_sp = calcNewFreeSpindles True t inst
                new_inst_sp = calcSpindleUse True t inst
@@ -1060,14 +1096,84 @@ availDisk t =
 iDsk :: Node -> Int
 iDsk t = truncate (tDsk t) - fDsk t
 
+-- | Returns state-of-world free memory on the node.
+-- | NOTE: This value is valid only before placement simulations.
+-- | TODO: Redefine this for memoy overcommitment.
+reportedFreeMem :: Node -> Int
+reportedFreeMem = fMem
+
+-- | Computes state-of-record free memory on the node.
+-- | TODO: Redefine this for memory overcommitment.
+recordedFreeMem :: Node -> Int
+recordedFreeMem t =
+  let total = tMem t
+      node = nMem t
+      inst = iMem t
+  in truncate total - node - inst
+
+-- | Computes the amount of missing memory on the node.
+-- NOTE: This formula uses free memory for calculations as opposed to
+-- used_memory in the definition, that's why it is the inverse.
+-- Explanations for missing memory (+) positive, (-) negative:
+-- (+) instances are using more memory that state-of-record
+--     - on KVM this might be due to the overhead per qemu process
+--     - on Xen manually upsized domains (xen mem-set)
+-- (+) on KVM non-qemu processes might be using more memory than what is
+--     reserved for node (no isolation)
+-- (-) on KVM qemu processes allocate memory on demand, thus an instance grows
+--     over its lifetime until it reaches state-of-record (+overhead)
+-- (-) on KVM KSM might be active
+-- (-) on Xen manually downsized domains (xen mem-set)
+missingMem :: Node -> Int
+missingMem t =
+  recordedFreeMem t - reportedFreeMem t
+
+-- | Computes the 'guaranteed' free memory, that is the minimum of what
+-- is reported by the node (available bytes) and our calculation based on
+-- instance sizes (our records), thus considering missing memory.
+-- NOTE 1: During placement simulations, the recorded memory changes, as
+-- instances are added/removed from the node, thus we have to calculate the
+-- missingMem (correction) before altering state-of-record and then
+-- use that correction to estimate state-of-world memory usage _after_
+-- the placements are done rather than doing min(record, world).
+-- NOTE 2: This is still only an approximation on KVM. As we shuffle instances
+-- during the simulation we are considering their state-of-record size, but
+-- in the real world the moves would shuffle parts of missing memory as well.
+-- Unfortunately as long as we don't have a more finegrained model that can
+-- better explain missing memory (split down based on root causes), we can't
+-- do better.
+-- NOTE 3: This is a hard limit based on available bytes and our bookkeeping.
+-- In case of memory overcommitment, both recordedFreeMem and reportedFreeMem
+-- would be extended by swap size on KVM or baloon size on Xen (their nominal
+-- and reported values).
+unallocatedMem :: Node -> Int
+unallocatedMem t =
+ let state_of_record = recordedFreeMem t
+ in state_of_record - max 0 (xMem t)
+
 -- | Computes the amount of available memory on a given node.
+-- Compared to unallocatedMem, this takes into account also memory reserved for
+-- secondary instances.
+-- NOTE: In case of memory overcommitment, there would be also an additional
+-- soft limit based on RAM size dedicated for instances and sum of
+-- state-of-record instance sizes (iMem): (tMem - nMem)*overcommit_ratio - iMem
 availMem :: Node -> Int
 availMem t =
-  let _f = fMem t
-      _l = rMem t
-  in if _f < _l
-       then 0
-       else _f - _l
+  let reserved = rMem t
+      unallocated = unallocatedMem t
+  in max 0 (unallocated - reserved)
+
+-- | Prospective memory stats after instance operation.
+prospectiveMem :: Node -> Instance
+               -> Bool       -- ^ Operation: True if add, False for remove.
+               -> (Int, Int) -- ^ Tuple (used_by_instances, guaranteed_free_mem)
+prospectiveMem node inst add =
+  let uses_mem = (Instance.usesMemory inst)
+      condOp = if add then incIf else decIf
+      new_i_mem = condOp uses_mem (iMem node) (Instance.mem inst)
+      new_node = node { iMem = new_i_mem }
+      new_free_mem = unallocatedMem new_node
+ in (new_i_mem, new_free_mem)
 
 -- | Computes the amount of available memory on a given node.
 availCpu :: Node -> Int
@@ -1078,10 +1184,6 @@ availCpu t =
        then _l - _u
        else 0
 
--- | The memory used by instances on a given node.
-iMem :: Node -> Int
-iMem t = truncate (tMem t) - nMem t - xMem t - fMem t
-
 -- * Node graph functions
 -- These functions do the transformations needed so that nodes can be
 -- represented as a graph connected by the instances that are replicated
@@ -1159,9 +1261,10 @@ showField t field =
     "nmem" -> printf "%5d" $ nMem t
     "xmem" -> printf "%5d" $ xMem t
     "fmem" -> printf "%5d" $ fMem t
+    "umem" -> printf "%5d" $ unallocatedMem t
     "imem" -> printf "%5d" $ iMem t
     "rmem" -> printf "%5d" $ rMem t
-    "amem" -> printf "%5d" $ fMem t - rMem t
+    "amem" -> printf "%5d" $ availMem t
     "tdsk" -> printf "%5.0f" $ tDsk t / 1024
     "fdsk" -> printf "%5d" $ fDsk t `div` 1024
     "tcpu" -> printf "%4.0f" $ tCpu t
@@ -1200,9 +1303,10 @@ showHeader field =
     "nmem" -> ("n_mem", True)
     "xmem" -> ("x_mem", True)
     "fmem" -> ("f_mem", True)
+    "umem" -> ("u_mem", True)
+    "amem" -> ("a_mem", True)
     "imem" -> ("i_mem", True)
     "rmem" -> ("r_mem", True)
-    "amem" -> ("a_mem", True)
     "tdsk" -> ("t_dsk", True)
     "fdsk" -> ("f_dsk", True)
     "tcpu" -> ("pcpu", True)
@@ -1289,7 +1393,7 @@ genAddTagsOpCode node tags = OpCodes.OpTagsSet
 -- | Constant holding the fields we're displaying by default.
 defaultFields :: [String]
 defaultFields =
-  [ "status", "name", "tmem", "nmem", "imem", "xmem", "fmem"
+  [ "status", "name", "tmem", "nmem", "imem", "xmem", "fmem", "umem"
   , "rmem", "tdsk", "fdsk", "tcpu", "ucpu", "pcnt", "scnt"
   , "pfmem", "pfdsk", "rcpu"
   , "cload", "mload", "dload", "nload" ]
