@@ -1,5 +1,5 @@
 {-# LANGUAGE TemplateHaskell, FunctionalDependencies, FlexibleContexts, CPP,
-             TypeFamilies, UndecidableInstances #-}
+             GeneralizedNewtypeDeriving, TypeFamilies, UndecidableInstances #-}
 -- {-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
 
 {-| Creates a client out of list of RPC server components.
@@ -43,14 +43,6 @@ module Ganeti.THH.HsRPC
   , mkRpcCalls
   ) where
 
--- The following macro is just a temporary solution for 2.12 and 2.13.
--- Since 2.14 cabal creates proper macros for all dependencies.
-#define MIN_VERSION_monad_control(maj,min,rev) \
-  (((maj)<MONAD_CONTROL_MAJOR)|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)<=MONAD_CONTROL_MINOR))|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)==MONAD_CONTROL_MINOR)&& \
-    ((rev)<=MONAD_CONTROL_REV)))
-
 import Control.Applicative
 import Control.Monad
 import Control.Monad.Base
@@ -74,28 +66,8 @@ import Ganeti.UDSServer
 -- result or the error.
 newtype RpcClientMonad a =
   RpcClientMonad { runRpcClientMonad :: ReaderT Client ResultG a }
-
-instance Functor RpcClientMonad where
-  fmap f = RpcClientMonad . fmap f . runRpcClientMonad
-
-instance Applicative RpcClientMonad where
-  pure = RpcClientMonad . pure
-  (RpcClientMonad f) <*> (RpcClientMonad k) = RpcClientMonad (f <*> k)
-
-instance Monad RpcClientMonad where
-  return = RpcClientMonad . return
-  (RpcClientMonad k) >>= f = RpcClientMonad (k >>= runRpcClientMonad . f)
-
-instance MonadBase IO RpcClientMonad where
-  liftBase = RpcClientMonad . liftBase
-
-instance MonadIO RpcClientMonad where
-  liftIO = RpcClientMonad . liftIO
-
-instance MonadError GanetiException RpcClientMonad where
-  throwError = RpcClientMonad . throwError
-  catchError (RpcClientMonad k) h =
-    RpcClientMonad (catchError k (runRpcClientMonad . h))
+  deriving (Functor, Applicative, Monad, MonadIO, MonadBase IO,
+            MonadError GanetiException)
 
 instance MonadBaseControl IO RpcClientMonad where
 #if MIN_VERSION_monad_control(1,0,0)
