@@ -96,6 +96,7 @@ module Ganeti.Utils
   , ensurePermissions
   , ordNub
   , isSubsequenceOf
+  , frequency
   ) where
 
 import Control.Applicative
@@ -818,3 +819,9 @@ isSubsequenceOf []    _                    = True
 isSubsequenceOf _     []                   = False
 isSubsequenceOf a@(x:a') (y:b) | x == y    = isSubsequenceOf a' b
                                | otherwise = isSubsequenceOf a b
+
+{-# ANN frequency "HLint: ignore Use alternative" #-}
+-- | Returns a list of tuples of elements and the number of times they occur
+-- in a list
+frequency :: Ord t => [t] -> [(Int, t)]
+frequency xs = map (\x -> (length x, head x)) . group . sort $ xs
