@@ -38,6 +38,7 @@ import Ganeti.Hs2Py.GenOpCodes
 import Ganeti.Hs2Py.ListConstants
 import Ganeti.THH.PyRPC
 import qualified Ganeti.WConfd.Core as WConfd
+import qualified Ganeti.Metad.ConfigCore as Metad
 
 import System.Environment (getArgs)
 import System.Exit (exitFailure)
@@ -52,6 +53,9 @@ main = do
     ["--wconfd-rpc"] -> putStrLn $
       $( genPyUDSRpcStubStr "ClientRpcStub" "WCONFD_SOCKET"
                             WConfd.exportedFunctions )
+    ["--metad-rpc"] -> putStrLn $
+      $( genPyUDSRpcStubStr "ClientRpcStub" "METAD_SOCKET"
+                            Metad.exportedFunctions )
     _ -> do
       hPutStrLn stderr "Usage: hs2py --opcodes\
                                   \| --constants\
