@@ -36,7 +36,6 @@ import OpenSSL
 
 import copy
 import unittest
-import operator
 import re
 import shutil
 import os
@@ -99,7 +98,7 @@ class TestClusterVerifySsh(unittest.TestCase):
       objects.Node(name="node50", group="aaa", offline=False,
                    master_candidate=True),
       ] + mygroupnodes
-    assert not utils.FindDuplicates(map(operator.attrgetter("name"), nodes))
+    assert not utils.FindDuplicates(n.name for n in nodes)
 
     (online, perhost, _) = fn(mygroupnodes, "my", nodes)
     self.assertEqual(online, ["node%s" % i for i in range(20, 26)])
@@ -126,7 +125,7 @@ class TestClusterVerifySsh(unittest.TestCase):
       objects.Node(name="node4", group="default", offline=True,
                    master_candidate=True),
       ]
-    assert not utils.FindDuplicates(map(operator.attrgetter("name"), nodes))
+    assert not utils.FindDuplicates(n.name for n in nodes)
 
     (online, perhost, _) = fn(nodes, "default", nodes)
     self.assertEqual(online, ["node2", "node3"])
@@ -806,6 +805,22 @@ class TestLUClusterSetParams(CmdlibTestCase):
 
     assert constants.HT_FAKE not in self.cluster.os_hvp["mocked_os"]
 
+  def testRemoveOsFromOsHvpList(self):
+    os_hvp = {
+        "mocked_os_1": {
+            constants.HT_FAKE: {
+                constants.HV_MIGRATION_MODE: constants.HT_MIGRATION_NONLIVE
+            }
+        },
+        "mocked_os_2": {} # This is the one that needs to be removed.
+    }
+
+    op = opcodes.OpClusterSetParams(os_hvp=os_hvp)
+    self.ExecOpCode(op)
+
+    assert (constants.HT_FAKE in self.cluster.os_hvp["mocked_os_1"] and
+            "mocked_os_2" not in self.cluster.os_hvp)
+
   def testDefaultOsHvp(self):
     os_hvp = {"mocked_os": constants.HVC_DEFAULTS.copy()}
     self.cluster.os_hvp = {"mocked_os": {}}
@@ -1232,8 +1247,13 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
         .Build()
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
-    self.mcpu.assertLogContainsRegex("Client certificate")
-    self.mcpu.assertLogContainsRegex("failed validation")
+    regexps = (
+      "Client certificate",
+      "failed validation",
+      "gnt-cluster renew-crypto --new-node-certificates",
+    )
+    for r in regexps:
+      self.mcpu.assertLogContainsRegex(r)
 
   def testVerifyNoMasterCandidateMap(self):
     client_cert = "client-cert-digest"
@@ -1247,6 +1267,8 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
     self.ExecOpCode(op)
     self.mcpu.assertLogContainsRegex(
       "list of master candidate certificates is empty")
+    self.mcpu.assertLogContainsRegex(
+      "gnt-cluster renew-crypto --new-node-certificates")
 
   def testVerifyNoSharingMasterCandidates(self):
     client_cert = "client-cert-digest"
@@ -1262,6 +1284,8 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
     self.ExecOpCode(op)
     self.mcpu.assertLogContainsRegex(
       "two master candidates configured to use the same")
+    self.mcpu.assertLogContainsRegex(
+      "gnt-cluster renew-crypto --new-node-certificates")
 
   def testVerifyMasterCandidateCertMismatch(self):
     client_cert = "client-cert-digest"
@@ -1274,6 +1298,8 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
     self.mcpu.assertLogContainsRegex("does not match its entry")
+    self.mcpu.assertLogContainsRegex(
+      "gnt-cluster renew-crypto --new-node-certificates")
 
   def testVerifyMasterCandidateUnregistered(self):
     client_cert = "client-cert-digest"
@@ -1286,6 +1312,8 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
     self.mcpu.assertLogContainsRegex("does not have an entry")
+    self.mcpu.assertLogContainsRegex(
+      "gnt-cluster renew-crypto --new-node-certificates")
 
   def testVerifyMasterCandidateOtherNodesCert(self):
     client_cert = "client-cert-digest"
@@ -1298,6 +1326,8 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
     self.mcpu.assertLogContainsRegex("using a certificate of another node")
+    self.mcpu.assertLogContainsRegex(
+      "gnt-cluster renew-crypto --new-node-certificates")
 
   def testNormalNodeStillInList(self):
     self._AddNormalNode()
@@ -1315,8 +1345,13 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
         .Build()
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
-    self.mcpu.assertLogContainsRegex("not a master candidate")
-    self.mcpu.assertLogContainsRegex("still listed")
+    regexps = (
+      "not a master candidate",
+      "still listed",
+      "gnt-cluster renew-crypto --new-node-certificates",
+    )
+    for r in regexps:
+      self.mcpu.assertLogContainsRegex(r)
 
   def testNormalNodeStealingMasterCandidateCert(self):
     self._AddNormalNode()
@@ -1332,9 +1367,13 @@ class TestLUClusterVerifyClientCerts(CmdlibTestCase):
         .Build()
     op = opcodes.OpClusterVerifyGroup(group_name="default", verbose=True)
     self.ExecOpCode(op)
-    self.mcpu.assertLogContainsRegex("not a master candidate")
-    self.mcpu.assertLogContainsRegex(
-      "certificate of another node which is master candidate")
+    regexps = (
+      "not a master candidate",
+      "certificate of another node which is master candidate",
+      "gnt-cluster renew-crypto --new-node-certificates",
+    )
+    for r in regexps:
+      self.mcpu.assertLogContainsRegex(r)
 
 
 class TestLUClusterVerifyGroupMethods(CmdlibTestCase):
@@ -1891,7 +1930,7 @@ class TestLUClusterVerifyGroupVerifyFiles(TestLUClusterVerifyGroupMethods):
       .AddSuccessfulNode(node4, {}) \
       .AddOfflineNode(node5) \
       .Build()
-    assert set(nvinfo.keys()) == set(map(operator.attrgetter("uuid"), nodeinfo))
+    assert set(nvinfo.keys()) == set(ni.uuid for ni in nodeinfo)
 
     lu._VerifyFiles(nodeinfo, self.master_uuid, nvinfo,
                     (files_all, files_opt, files_mc, files_vm))
@@ -2326,7 +2365,30 @@ class TestLUClusterVerifyGroupHooksCallBack(TestLUClusterVerifyGroupMethods):
 
 
 class TestLUClusterVerifyDisks(CmdlibTestCase):
+
   def testVerifyDisks(self):
+    self.cfg.AddNewInstance(uuid="tst1.inst.corp.google.com",
+                            disk_template=constants.DT_PLAIN)
+    op = opcodes.OpClusterVerifyDisks()
+    result = self.ExecOpCode(op)
+
+    self.assertEqual(1, len(result["jobs"]))
+
+  def testVerifyDisksExt(self):
+    self.cfg.AddNewInstance(uuid="tst1.inst.corp.google.com",
+                            disk_template=constants.DT_EXT)
+    self.cfg.AddNewInstance(uuid="tst2.inst.corp.google.com",
+                            disk_template=constants.DT_EXT)
+    op = opcodes.OpClusterVerifyDisks()
+    result = self.ExecOpCode(op)
+
+    self.assertEqual(0, len(result["jobs"]))
+
+  def testVerifyDisksMixed(self):
+    self.cfg.AddNewInstance(uuid="tst1.inst.corp.google.com",
+                            disk_template=constants.DT_EXT)
+    self.cfg.AddNewInstance(uuid="tst2.inst.corp.google.com",
+                            disk_template=constants.DT_PLAIN)
     op = opcodes.OpClusterVerifyDisks()
     result = self.ExecOpCode(op)
 
@@ -2342,7 +2404,7 @@ class TestLUClusterRenewCrypto(CmdlibTestCase):
     self._client_node_cert = self._CreateTempFile()
     shutil.copy(testutils.TestDataFilename("cert2.pem"), self._client_node_cert)
     self._client_node_cert_digest = \
-        "BF:24:F7:57:50:60:43:87:83:E3:0D:7E:EF:DD:14:6C:13:43:20:4E"
+        "30:AF:82:D0:00:1C:F2:99:DE:A8:6D:31:7F:C9:D5:46:70:07:EC:4F"
 
   def tearDown(self):
     super(TestLUClusterRenewCrypto, self).tearDown()
