@@ -133,27 +133,74 @@ class TestNodeVerify(testutils.GanetiTestCase):
 
   def testMasterIPLocalhost(self):
     # this a real functional test, but requires localhost to be reachable
-    local_data = (netutils.Hostname.GetSysName(),
-                  constants.IP4_ADDRESS_LOCALHOST)
+    my_name = netutils.Hostname.GetSysName()
+    local_data = (my_name,constants.IP4_ADDRESS_LOCALHOST, [my_name])
     result = backend.VerifyNode({constants.NV_MASTERIP: local_data},
-                                None, {}, {}, {})
+                                None, {})
     self.failUnless(constants.NV_MASTERIP in result,
                     "Master IP data not returned")
-    self.failUnless(result[constants.NV_MASTERIP], "Cannot reach localhost")
+    self.failUnless(result[constants.NV_MASTERIP],
+                    "Cannot reach localhost")
+
+  def testMasterIPSkipTest(self):
+    # this a real functional test, but requires localhost to be reachable
+    local_data = (netutils.Hostname.GetSysName(),
+                  constants.IP4_ADDRESS_LOCALHOST, [])
+    result = backend.VerifyNode({constants.NV_MASTERIP: local_data},
+                                None, {})
+    self.failUnless(constants.NV_MASTERIP in result,
+                    "Master IP data not returned")
+    self.failUnless(result[constants.NV_MASTERIP] == None,
+                    "Test ran by non master candidate")
 
   def testMasterIPUnreachable(self):
     # Network 192.0.2.0/24 is reserved for test/documentation as per
     # RFC 5737
-    bad_data =  ("master.example.com", "192.0.2.1")
+    my_name = "master.example.com"
+    bad_data =  (my_name, "192.0.2.1", [my_name])
     # we just test that whatever TcpPing returns, VerifyNode returns too
     netutils.TcpPing = lambda a, b, source=None: False
     result = backend.VerifyNode({constants.NV_MASTERIP: bad_data},
-                                None, {}, {}, {})
+                                None, {})
     self.failUnless(constants.NV_MASTERIP in result,
                     "Master IP data not returned")
     self.failIf(result[constants.NV_MASTERIP],
                 "Result from netutils.TcpPing corrupted")
 
+  def testVerifyNodeNetTestMissingSelf(self):
+    my_name = netutils.Hostname.GetSysName()
+    local_data = ([('n1.test.com', "any", "any")], [my_name])
+    result = backend.VerifyNode({constants.NV_NODENETTEST: local_data},
+                                None, {})
+
+    self.failUnless(constants.NV_NODENETTEST in result,
+                    "NodeNetTest data not returned")
+    self.failUnless(my_name in result[constants.NV_NODENETTEST],
+                    "Missing failure in net test")
+
+  def testVerifyNodeNetTest(self):
+    my_name = netutils.Hostname.GetSysName()
+    local_data = ([(my_name, "any", "any")], [my_name])
+
+    # we just test that whatever TcpPing returns, VerifyNode returns too
+    netutils.TcpPing = lambda a, b, source=None: True
+    result = backend.VerifyNode({constants.NV_NODENETTEST: local_data},
+                                None, {})
+
+    self.failUnless(constants.NV_NODENETTEST in result,
+                    "NodeNetTest data not returned")
+    self.failUnless(result[constants.NV_NODENETTEST] == {},
+                    "NodeNetTest failed")
+
+  def testVerifyNodeNetSkipTest(self):
+    local_data = ([('n1.test.com', "any", "any")], [])
+    result = backend.VerifyNode({constants.NV_NODENETTEST: local_data},
+                                None, {})
+    self.failUnless(constants.NV_NODENETTEST in result,
+                    "NodeNetTest data not returned")
+    self.failUnless(result[constants.NV_NODENETTEST] == {},
+                    "Test ran by non master candidate")
+
   def testVerifyHvparams(self):
     test_hvparams = {constants.HV_XEN_CMD: constants.XEN_CMD_XL}
     test_what = {constants.NV_HVPARAMS: \
@@ -1031,6 +1078,8 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
         self._ssh_file_manager.GetAllNodeNames
     self._ssconf_mock.GetOnlineNodeList.side_effect = \
         self._ssh_file_manager.GetAllNodeNames
+    self._ssconf_mock.GetMasterNode.side_effect = \
+        self._ssh_file_manager.GetMasterNodeName
 
   def _TearDownTestData(self):
     os.remove(self._pub_key_file)
@@ -1055,6 +1104,7 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
     backend._GenerateNodeSshKey(
         test_node_uuid, test_node_name,
         self._ssh_file_manager.GetSshPortMap(self._SSH_PORT),
+        "rsa", 2048,
         pub_key_file=self._pub_key_file,
         ssconf_store=self._ssconf_mock,
         noded_cert_file=self.noded_cert_file,
@@ -1236,8 +1286,7 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
         is_potential_master_candidate, is_master_candidate,
         is_master)
 
-    self.assertRaises(
-        AssertionError, backend.AddNodeSshKey, new_node_uuid, new_node_name,
+    backend.AddNodeSshKey(new_node_uuid, new_node_name,
         self._potential_master_candidates,
         to_authorized_keys=is_master_candidate,
         to_public_keys=is_potential_master_candidate,
@@ -1255,8 +1304,7 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
     (node_list, key_map) = self._SetupNodeBulk(
         num_nodes, self._GetNewNumberedNormalNode)
 
-    self.assertRaises(
-        AssertionError, backend.AddNodeSshKeyBulk, node_list,
+    backend.AddNodeSshKeyBulk(node_list,
         self._potential_master_candidates,
         pub_key_file=self._pub_key_file,
         ssconf_store=self._ssconf_mock,
@@ -1365,6 +1413,37 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
     self.assertEqual(1,
         len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_name)))
 
+  def testRemoveMasterCandidateBulk(self):
+    node_list = []
+    key_map = {}
+    for node_name, (node_uuid, node_key, _, _, _) in \
+        self._ssh_file_manager.GetAllMasterCandidates()[:3]:
+      node_list.append(backend.SshRemoveNodeInfo(uuid=node_uuid,
+                                                 name=node_name,
+                                                 from_authorized_keys=True,
+                                                 from_public_keys=True,
+                                                 clear_authorized_keys=True,
+                                                 clear_public_keys=True))
+      key_map[node_name] = node_key
+
+    backend.RemoveNodeSshKeyBulk(node_list,
+                                 self._master_candidate_uuids,
+                                 self._potential_master_candidates,
+                                 pub_key_file=self._pub_key_file,
+                                 ssconf_store=self._ssconf_mock,
+                                 noded_cert_file=self.noded_cert_file,
+                                 run_cmd_fn=self._run_cmd_mock)
+
+    for node_info in node_list:
+      self._ssh_file_manager.AssertNoNodeHasPublicKey(
+          node_info.uuid, key_map[node_info.name])
+      self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+          [node_info.name], key_map[node_info.name])
+      self.assertEqual(0,
+          len(self._ssh_file_manager.GetPublicKeysOfNode(node_info.name)))
+      self.assertEqual(1,
+          len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_info.name)))
+
   def testRemovePotentialMasterCandidate(self):
     (node_name, node_info) = \
         self._ssh_file_manager.GetAllPurePotentialMasterCandidates()[0]
@@ -1383,12 +1462,44 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
 
     self._ssh_file_manager.AssertNoNodeHasPublicKey(
         node_info.uuid, node_info.key)
-    self._ssh_file_manager.AssertNoNodeHasAuthorizedKey(node_info.key)
+    self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+        [node_name], node_info.key)
     self.assertEqual(0,
         len(self._ssh_file_manager.GetPublicKeysOfNode(node_name)))
-    self.assertEqual(0,
+    self.assertEqual(1,
         len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_name)))
 
+  def testRemovePotentialMasterCandidateBulk(self):
+    node_list = []
+    key_map = {}
+    for node_name, (node_uuid, node_key, _, _, _) in \
+        self._ssh_file_manager.GetAllPurePotentialMasterCandidates()[:3]:
+      node_list.append(backend.SshRemoveNodeInfo(uuid=node_uuid,
+                                                 name=node_name,
+                                                 from_authorized_keys=False,
+                                                 from_public_keys=True,
+                                                 clear_authorized_keys=True,
+                                                 clear_public_keys=True))
+      key_map[node_name] = node_key
+
+    backend.RemoveNodeSshKeyBulk(node_list,
+                                 self._master_candidate_uuids,
+                                 self._potential_master_candidates,
+                                 pub_key_file=self._pub_key_file,
+                                 ssconf_store=self._ssconf_mock,
+                                 noded_cert_file=self.noded_cert_file,
+                                 run_cmd_fn=self._run_cmd_mock)
+
+    for node_info in node_list:
+      self._ssh_file_manager.AssertNoNodeHasPublicKey(
+          node_info.uuid, key_map[node_info.name])
+      self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+          [node_info.name], key_map[node_info.name])
+      self.assertEqual(0,
+          len(self._ssh_file_manager.GetPublicKeysOfNode(node_info.name)))
+      self.assertEqual(1,
+          len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_info.name)))
+
   def testRemoveNormalNode(self):
     node_name, node_info = self._ssh_file_manager.GetAllNormalNodes()[0]
 
@@ -1406,12 +1517,77 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
 
     self._ssh_file_manager.AssertNoNodeHasPublicKey(
         node_info.uuid, node_info.key)
-    self._ssh_file_manager.AssertNoNodeHasAuthorizedKey(node_info.key)
+    self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+        [node_name], node_info.key)
     self.assertEqual(0,
         len(self._ssh_file_manager.GetPublicKeysOfNode(node_name)))
-    self.assertEqual(0,
+    self.assertEqual(1,
         len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_name)))
 
+  def testRemoveNormalNodeBulk(self):
+    node_list = []
+    key_map = {}
+    for node_name, (node_uuid, node_key, _, _, _) in \
+        self._ssh_file_manager.GetAllNormalNodes()[:3]:
+      node_list.append(backend.SshRemoveNodeInfo(uuid=node_uuid,
+                                                 name=node_name,
+                                                 from_authorized_keys=False,
+                                                 from_public_keys=False,
+                                                 clear_authorized_keys=True,
+                                                 clear_public_keys=True))
+      key_map[node_name] = node_key
+
+    backend.RemoveNodeSshKeyBulk(node_list,
+                                 self._master_candidate_uuids,
+                                 self._potential_master_candidates,
+                                 pub_key_file=self._pub_key_file,
+                                 ssconf_store=self._ssconf_mock,
+                                 noded_cert_file=self.noded_cert_file,
+                                 run_cmd_fn=self._run_cmd_mock)
+
+    for node_info in node_list:
+      self._ssh_file_manager.AssertNoNodeHasPublicKey(
+          node_info.uuid, key_map[node_info.name])
+      self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+          [node_info.name], key_map[node_info.name])
+      self.assertEqual(0,
+          len(self._ssh_file_manager.GetPublicKeysOfNode(node_info.name)))
+      self.assertEqual(1,
+          len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_info.name)))
+
+  def testRemoveDiverseNodesBulk(self):
+    node_list = []
+    key_map = {}
+    for node_name, (node_uuid, node_key, is_potential_master_candidate,
+                    is_master_candidate, _) in \
+        self._ssh_file_manager.GetAllNodesDiverse()[:3]:
+      node_list.append(backend.SshRemoveNodeInfo(
+          uuid=node_uuid,
+          name=node_name,
+          from_authorized_keys=is_master_candidate,
+          from_public_keys=is_potential_master_candidate,
+          clear_authorized_keys=True,
+          clear_public_keys=True))
+      key_map[node_name] = node_key
+
+    backend.RemoveNodeSshKeyBulk(node_list,
+                                 self._master_candidate_uuids,
+                                 self._potential_master_candidates,
+                                 pub_key_file=self._pub_key_file,
+                                 ssconf_store=self._ssconf_mock,
+                                 noded_cert_file=self.noded_cert_file,
+                                 run_cmd_fn=self._run_cmd_mock)
+
+    for node_info in node_list:
+      self._ssh_file_manager.AssertNoNodeHasPublicKey(
+          node_info.uuid, key_map[node_info.name])
+      self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+          [node_info.name], key_map[node_info.name])
+      self.assertEqual(0,
+          len(self._ssh_file_manager.GetPublicKeysOfNode(node_info.name)))
+      self.assertEqual(1,
+          len(self._ssh_file_manager.GetAuthorizedKeysOfNode(node_info.name)))
+
   def testDemoteMasterCandidateToPotentialMasterCandidate(self):
     node_name, node_info = self._ssh_file_manager.GetAllMasterCandidates()[0]
     self._ssh_file_manager.SetOrAddNode(
@@ -1456,7 +1632,8 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
 
     self._ssh_file_manager.AssertNoNodeHasPublicKey(
         node_info.uuid, node_info.key)
-    self._ssh_file_manager.AssertNoNodeHasAuthorizedKey(node_info.key)
+    self._ssh_file_manager.AssertNodeSetOnlyHasAuthorizedKey(
+        [node_name], node_info.key)
 
   def _GetReducedOnlineNodeList(self):
     """'Randomly' mark some nodes as offline."""
@@ -1579,8 +1756,9 @@ class TestAddRemoveGenerateNodeSshKey(testutils.GanetiTestCase):
         noded_cert_file=self.noded_cert_file,
         run_cmd_fn=self._run_cmd_mock)
 
+    master_node = self._ssh_file_manager.GetMasterNodeName()
     for node in self._all_nodes:
-      if node == new_node_name:
+      if node in [new_node_name, master_node]:
         self.assertTrue(self._ssh_file_manager.NodeHasAuthorizedKey(
           node, new_node_key))
       else:
@@ -1825,8 +2003,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._read_file_mock = self._read_file_patcher.start()
     self._read_file_mock.return_value = self._NODE1_KEYS[0]
     self.tmpdir = tempfile.mkdtemp()
-    self.pub_key_file = os.path.join(self.tmpdir, "pub_key_file")
-    open(self.pub_key_file, "w").close()
+    self.pub_keys_file = os.path.join(self.tmpdir, "pub_keys_file")
+    open(self.pub_keys_file, "w").close()
 
   def tearDown(self):
     super(testutils.GanetiTestCase, self).tearDown()
@@ -1841,7 +2019,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._query_mock.return_value = self._PUB_KEY_RESULT
     result = backend._VerifySshSetup(self._NODE_STATUS_LIST,
                                      self._NODE1_NAME,
-                                     pub_key_file=self.pub_key_file)
+                                     "dsa",
+                                     ganeti_pub_keys_file=self.pub_keys_file)
     self.assertEqual(result, [])
 
   def testMissingKey(self):
@@ -1852,7 +2031,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._query_mock.return_value = pub_key_missing
     result = backend._VerifySshSetup(self._NODE_STATUS_LIST,
                                      self._NODE1_NAME,
-                                     pub_key_file=self.pub_key_file)
+                                     "dsa",
+                                     ganeti_pub_keys_file=self.pub_keys_file)
     self.assertTrue(self._NODE2_UUID in result[0])
 
   def testUnknownKey(self):
@@ -1863,7 +2043,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._query_mock.return_value = pub_key_missing
     result = backend._VerifySshSetup(self._NODE_STATUS_LIST,
                                      self._NODE1_NAME,
-                                     pub_key_file=self.pub_key_file)
+                                     "dsa",
+                                     ganeti_pub_keys_file=self.pub_keys_file)
     self.assertTrue("unkownnodeuuid" in result[0])
 
   def testMissingMasterCandidate(self):
@@ -1874,7 +2055,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._query_mock.return_value = self._PUB_KEY_RESULT
     result = backend._VerifySshSetup(self._NODE_STATUS_LIST,
                                      self._NODE1_NAME,
-                                     pub_key_file=self.pub_key_file)
+                                     "dsa",
+                                     ganeti_pub_keys_file=self.pub_keys_file)
     self.assertTrue(self._NODE1_UUID in result[0])
 
   def testSuperfluousNormalNode(self):
@@ -1885,7 +2067,8 @@ class TestVerifySshSetup(testutils.GanetiTestCase):
     self._query_mock.return_value = self._PUB_KEY_RESULT
     result = backend._VerifySshSetup(self._NODE_STATUS_LIST,
                                      self._NODE1_NAME,
-                                     pub_key_file=self.pub_key_file)
+                                     "dsa",
+                                     ganeti_pub_keys_file=self.pub_keys_file)
     self.assertTrue(self._NODE3_UUID in result[0])
 
 
